from typing import Optional

from cpl_core.console import Console
from cpl_query.extension import List
from multi_install.abc.menu_service_abc import MenuServiceABC
from multi_install.config.os_settings import OSSettings
from multi_install.model.main_menu_enum import MainMenuEnum


class MenuService(MenuServiceABC):

    def __init__(self, os_settings: OSSettings):
        self._os_settings = os_settings
        
        self._os: Optional[str] = None
    
    def run(self):
        is_end = False
        while not is_end:
            is_end = self.select_os()
            if not is_end:
                is_end = self.main_menu()
                
            if is_end:
                Console.write_line()
        
    def select_os(self) -> bool:
        Console.clear()
        options = List(str, self._os_settings.operating_systems.copy())
        options.append('Exit')
        Console.write_line('Select option:')
        options.for_each(lambda o: Console.write_line(f'[{options.index(o)+1}] {o}'))
        os = 0
        try:
            os = int(Console.read('\n:'))-1
        except ValueError:
            Console.error('Input value must be int!')
            return True
        
        if os == options.index('Exit'):
            return True
        
        self._os = os
        return False
        
    def main_menu(self) -> bool:
        Console.clear()
        options = List(str, [
            MainMenuEnum.install_collection.value,
            MainMenuEnum.uninstall_collection.value,
            MainMenuEnum.add_collection.value,
            MainMenuEnum.remove_collection.value,
            
            MainMenuEnum.install_application.value,
            MainMenuEnum.uninstall_application.value,
            MainMenuEnum.add_application.value,
            MainMenuEnum.remove_application.value,
            
            MainMenuEnum.back.value,
            MainMenuEnum.exit.value
        ])
        Console.write_line('Select option:')
        options.for_each(lambda o: Console.write_line(f'[{options.index(o)+1}] {o}'))
        selected = 0
        try:
            selected = int(Console.read('\n:'))-1
        except ValueError:
            Console.error('Input value must be int!')
            return True
        
        # collection menu
        if selected == options.index(MainMenuEnum.install_collection.value): pass
        elif selected == options.index(MainMenuEnum.uninstall_collection.value): pass
        elif selected == options.index(MainMenuEnum.add_collection.value): pass
        elif selected == options.index(MainMenuEnum.remove_collection.value): pass
        
        # application menu
        elif selected == options.index(MainMenuEnum.install_application.value): pass
        elif selected == options.index(MainMenuEnum.uninstall_application.value): pass
        elif selected == options.index(MainMenuEnum.add_application.value): pass
        elif selected == options.index(MainMenuEnum.remove_application.value): pass
        
        elif selected == options.index(MainMenuEnum.back.value):
            return
        elif selected == options.index(MainMenuEnum.exit.value):
            return True
            
        else:
            raise Exception(f'Unknown option: {selected}')
        
        return False
